@@ -1,6 +1,5 @@
 //! Common code used in hRPC code generation.
 use http::{header::HeaderName, HeaderValue};
-use std::collections::HashMap;
 
 #[doc(inline)]
 pub use async_trait::async_trait;
@@ -9,6 +8,8 @@ pub use bytes;
 #[doc(hidden)]
 pub use futures_util;
 #[doc(hidden)]
+pub use tracing;
+#[doc(hidden)]
 pub use url;
 
 #[doc(hidden)]
@@ -32,8 +33,9 @@ pub mod client;
 #[cfg(feature = "server")]
 pub mod server;
 
-type HeaderMap = HashMap<HeaderName, HeaderValue>;
+use http::HeaderMap;
 
+#[derive(Debug, Clone)]
 /// A hRPC request.
 pub struct Request<T> {
     message: T,
@@ -50,10 +52,7 @@ impl<T> Request<T> {
             header_map: {
                 #[allow(clippy::mutable_key_type)]
                 let mut map: HeaderMap = HeaderMap::with_capacity(1);
-                map.insert(
-                    "content-type".parse().unwrap(),
-                    "application/hrpc".parse().unwrap(),
-                );
+                map.insert("content-type", "application/hrpc".parse().unwrap());
                 map
             },
         }
